import { ElementStyles, ElementViewTemplate, FASTElement, PartialFASTElementDefinition } from "@microsoft/fast-element";
import { ComponentPresentation } from "../design-system";
import { Registry } from "../di";
/**
 * An element definition used to define a FoundationElement when registered through the design
 * system registry.
 * @alpha
 */
export declare type FoundationElementDefinition = Omit<PartialFASTElementDefinition, "name"> & {
    /**
     * The non-prefixed name of the component.
     */
    baseName: string;
    /**
     * The element constructor
     */
    type: typeof FASTElement;
};
/**
 * A set of properties which the component consumer can override during the element registration process.
 * @alpha
 */
export declare type OverrideFoundationElementDefinition = Partial<Omit<FoundationElementDefinition, "type">> & {
    prefix?: string;
};
/**
 * Defines a foundation element class that:
 * 1. Connects the element to its ComponentPresentation
 * 2. Allows resolving the element template from the instance or ComponentPresentation
 * 3. Allows resolving the element styles from the instance or ComponentPresentation
 *
 * @alpha
 */
export declare class FoundationElement extends FASTElement {
    private container;
    private _presentation;
    /**
     * A property which resolves the ComponentPresentation instance
     * for the current component.
     */
    protected get $presentation(): ComponentPresentation;
    /**
     * Sets the template of the element instance. When undefined,
     * the element will attempt to resolve the template from
     * the $fastProvider
     */
    template: ElementViewTemplate | void | null;
    protected templateChanged(): void;
    /**
     * Sets the default styles for the element instance. When undefined,
     * the element will attempt to resolve default styles from
     * the $fastProvider
     */
    styles: ElementStyles | void | null;
    protected stylesChanged(): void;
    /**
     * The connected callback for this FASTElement.
     * @remarks
     * This method is invoked by the platform whenever this FoundationElement
     * becomes connected to the document.
     */
    connectedCallback(): void;
    /**
     * Creates an element registry configuration function with a set of element definition defaults.
     * @param elementDefinition - The definition of the element to create the registry configuration
     * function for.
     */
    static configuration(elementDefinition: FoundationElementDefinition): (overrideDefinition?: OverrideFoundationElementDefinition) => Registry;
}
